#!/usr/bin/perl -w
#
# Vampire the Masquerade - Web Sheet Generator
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2

use CGI qw(:standard); 						# Common Gateway Interface
use CGI::Carp qw(fatalsToBrowser);				# Display errors to screen
use strict;							# Strict Variable Enforcement


###########################
# Data Members

###############
# Settings

my $root = 'http://www.ethereal-realms.org/WoD/';		# Root for images

my %loc  = ('VTM'	=> 'masquerade.gif',			# Location of images
	    'VDA'	=> 'darkages.gif',
	    'Attribute'	=> 'attributebar.gif',
	    'Primary'	=> 'abilitybar.gif',
	    'Dicipline' => 'disciplinebar.gif',
	    'Advantage' => 'advantagebar.gif',
	    'Merits'	=> 'meritflawbar.gif',
	    'Rituals'	=> 'derangementritualbar.gif',
	    'Base'	=> 'basebar.gif', 
	    'Five'	=> 'map',
	    'Ten'	=> 'mapa',
	    'Ext' 	=> '.gif',
	    'Willpower'	=> 'wpsquares.gif',
	    'Health'	=> 'healthbar.gif',
	    'Combat'	=> 'combar.gif',
	    'CombatBar' => 'combatbar.gif',
	    'Exp'	=> 'expbox.gif');


###############
# Program

my $clan;							# Clan
my $concept;							# Concept of character
my $demeanor;							# Demeanor of character
my $humanity;							# Humanity
my $history;							# History of person
my $name;							# Name of character
my $nature;							# Nature of character
my $pathname;							# Path Name
my $pathtitle;							# Precusor
my $picture;							# Associated picture
my $willpower;							# Willpower

my $count = 0;							# Counter
my $current;							# Current value
my $game;							# Game number

my $lmost;							# Most elements 
my $ldiciplines;						# Diciplines
my $lbackgrounds;						# Backgrounds
my $ltraits;							# Traits
my $lvirtues;							# Virtues

my @derangements;						# Known derangements
my @flaws;							# Known flaws
my @merits;							# Known merits
my @rituals;							# Known rituals

my @names;							# Array of names
my @values;							# Array of values

my %physical;							# Physical attributes
my %social;							# Social attributes
my %mental;							# Mental attributes
my %talents;							# Talents abilities
my %skills;							# Skills abilities
my %knowledges;							# Knowledges abilities
my %diciplines;							# Known diciplines
my %backgrounds;						# Known backgrounds
my %traits;							# Other traits
my %virtues;							# Virtues


###########################
# Program area

	# Create CGI Handle
	my $cgi = new CGI;

	# Make sure script was properly called
	unless (defined($cgi->param('GAME'))) { die('Improper call'); }


	# Handling of scrolling lists
	# Retreive values
	@flaws        = $cgi->param('FLAWS');
	@merits       = $cgi->param('MERITS');
	@derangements = $cgi->param('DERANGEMENTS');
	@rituals      = $cgi->param('RITUALS');

	# Handling of singular values
	# Retreive values
	$clan         = $cgi->param('CLAN');
	$concept      = $cgi->param('CONCEPT');
	$demeanor     = $cgi->param('DEMEANOR');
	$history      = $cgi->param('HISTORY');	
	$name         = $cgi->param('NAME');
	$nature       = $cgi->param('NATURE');
	$pathname     = $cgi->param('PATH');
	$pathtitle    = $cgi->param('PATHNAME');
	$picture      = $cgi->param('PICTURE');


	# Handling of mish mash produced from graphics
	# Retreive values	
	@names        = split(/::/, $cgi->param('NAMES'));
	@values       = split(/::/, $cgi->param('VALUES'));

	# Cycle throught
	foreach $current (@names)
	{
		# Remove Underscores
		$current =~ s/_/ /gs;

		# Determine breed
		if    ($current =~ /^ATP(.+)/)			# Physical
		{ $physical{$1} = $values[$count]; }

		elsif ($current =~ /^ATS(.+)/)			# Social
		{ $social{$1} = $values[$count]; }

		elsif ($current =~ /^ATM(.+)/)			# Mental
		{ $mental{$1} = $values[$count]; }

		elsif ($current =~ /^PRT(.+)/)			# Talents
		{ $talents{$1} = $values[$count]; }

		elsif ($current =~ /^PRS(.+)/)			# Skills
		{ $skills{$1} = $values[$count]; }

		elsif ($current =~ /^PRK(.+)/)			# Knowledges
		{ $knowledges{$1} = $values[$count]; }

		elsif ($current =~ /^DIC(.+)/)			# Diciplins
		{ $diciplines{$1} = $values[$count]; }

		elsif ($current =~ /^BKG(.+)/)			# Backgrouns
		{ $backgrounds{$1} = $values[$count]; }

		elsif ($current =~ /^OTS(.+)/)			# Backgrouns
		{ $traits{$1} = $values[$count]; }

		elsif ($current =~ /^VIR(.+)/)			# Virtues
		{ $virtues{$1} = $values[$count]; }

		elsif ($current =~ /^WIL(.+)/)			# Willpower
		{ $willpower = $values[$count]; }

		elsif ($current =~ /^HUM(.+)/)			# Path or Humanity
		{ $humanity = $values[$count]; }

		# Increment for next pass
		$count++;
	}	


	# HTML Handler
	# File and HTML header
	print $cgi->header();

	# Title
	# Vampire the Masquerade
	if ($game == 0) 
	{ 
		print $cgi->start_html("Vampire: The Masquerade: $name"), "\n";
		print "<CENTER><IMG SRC=\"$root$loc{VTM}\"></CENTER>\n<BR>\n<BR>\n\n"; 

	}

	# Darkages
	elsif ($game == 1) 
	{ 
		print $cgi->start_html("Vampire: Dark Ages: $name"), "\n";
		print "<CENTER><IMG SRC=\"$root$loc{VDA}\"></CENTER>\n<BR>\n<BR>\n\n";
	}

	
	# Primary player information
	print 	"<CENTER><TABLE WIDTH=\"95%\">\n",
		" <TR>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Name</FONT></B></TD>\n",
		"  <TD ALIGN=\"right\">$name<TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Nature</FONT></B></TD>\n",
		"  <TD ALIGN=\"right\">$nature<TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Clan</FONT></B></TD>\n",
		"  <TD ALIGN=\"right\">$clan<TD>\n",
		" </TR>\n",
		" <TR>\n",
		"  <TD>&nbsp</TD>",
		"  <TD>&nbsp<TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Demeanor</FONT></B></TD>\n",
		"  <TD ALIGN=\"right\">$demeanor<TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Concept</FONT></B></TD>\n",
		"  <TD ALIGN=\"right\">$concept<TD>\n",
		" </TR>\n",
		"</TABLE></CENTER>\n";

	# Attributes
	print 	"<CENTER><TABLE WIDTH=\"95%\">\n";
 	print 	" <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{Attribute}\"><BR></TD></TR>\n\n",
		" <TR>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Physical</B></TD>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Social</B></TD>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Mental</B></TD>\n",
		" </TR>\n",
		" <TR>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Strength</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$physical{Strength}$loc{Ext}\"></TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Charisma</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$social{Charisma}$loc{Ext}\"></TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Perception</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$mental{Perception}$loc{Ext}\"></TD>\n",
		" </TR>\n",
		" <TR>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Dexterity</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$physical{Dexterity}$loc{Ext}\"></TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Manipulation</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$social{Manipulation}$loc{Ext}\"></TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Intelligence</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$mental{Intelligence}$loc{Ext}\"></TD>\n",
		" </TR>\n",
		" <TR>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Stamina</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$physical{Stamina}$loc{Ext}\"></TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Apperance</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$social{Apperance}$loc{Ext}\"></TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">Wits</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$mental{Wits}$loc{Ext}\"></TD>\n",
		" </TR>\n";

	# Primary abilities
	# Graphic and headers
 	print 	" <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{Primary}\"><BR></TD></TR>\n\n",
		" <TR>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Talents</B></TD>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Skills</B></TD>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Mental</B></TD>\n",
		" </TR>\n";

	# Retreive keys
	my @talents    = sort(keys(%talents)); 
	my @skills     = sort(keys(%skills));
	my @knowledges = sort(keys(%knowledges));

	# Reset counter
	$count         = 0;

	# Cycle trhought
	foreach $current (@talents)
	{
	  print " <TR>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">$talents[$count]</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$talents{$talents[$count]}$loc{Ext}\"></TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">$skills[$count]</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$skills{$skills[$count]}$loc{Ext}\"></TD>\n",
		"  <TD><B><FONT COLOR=\"#3F3FFF\">$knowledges[$count]</FONT></B></TD>\n",
		"  <TD><IMG SRC=\"$root$loc{Ten}$knowledges{$knowledges[$count]}$loc{Ext}\"></TD>\n",
		" </TR>\n";

	 	# Increment
		$count++;
	}
	

	# Advantages
	# Graphic and headers
 	print 	" <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{Advantage}\"><BR></TD></TR>\n\n",
		" <TR>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Diciplines</B></TD>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Backgrounds</B></TD>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Virtues</B></TD>\n",
		" </TR>\n";

	# Specific members
	my @diciplines;
	my @backgrounds;
	my @traits;
	my @virtues;

	# Only what is specified
	foreach $current (keys(%diciplines))  { if ($diciplines{$current}  > 0) { push(@diciplines, $current); }}
	foreach $current (keys(%backgrounds)) { if ($backgrounds{$current} > 0) { push(@backgrounds, $current); }}
	foreach $current (keys(%traits))      { if ($traits{$current}      > 0) { push(@traits, $current); }}
	foreach $current (keys(%virtues))     { if ($virtues{$current}     > 0) { push(@virtues, $current); }}


	# Retreive lengths
	$ldiciplines  = @diciplines;
	$lbackgrounds = @backgrounds;
	$ltraits      = @traits;
	$lvirtues     = @virtues;

	# Which is longest
	$lmost = ($ldiciplines > $lbackgrounds) ? $ldiciplines : $lbackgrounds;
	$lmost = ($lmost > $lvirtues) ? $lmost : $lvirtues;

	# Cycle through
	for ($count=0; $count < $lmost; $count++)
	{
		# Beginning of row
		print " <TR>\n";		

		# Diciplines
		if ($count < $ldiciplines)
		{
		  print "  <TD><B><FONT COLOR=\"#3F3FFF\">$diciplines[$count]</FONT></B></TD>\n",
                        "  <TD><IMG SRC=\"$root$loc{Ten}$diciplines{$diciplines[$count]}$loc{Ext}\"></TD>\n",
		}
		else
		{ print "  <TD>&nbsp;</TD>\n  <TD>&nbsp;</TD>\n"; }

		# Backgrounds
		if ($count < $lbackgrounds)
		{
		  print "  <TD><B><FONT COLOR=\"#3F3FFF\">$backgrounds[$count]</FONT></B></TD>\n",
                        "  <TD><IMG SRC=\"$root$loc{Ten}$backgrounds{$backgrounds[$count]}$loc{Ext}\"></TD>\n",
		}
		else
		{ print "  <TD>&nbsp;</TD>\n  <TD>&nbsp;</TD>\n"; }

		# Virtues
		if ($count < $lvirtues)
		{
		  print "  <TD><B><FONT COLOR=\"#3F3FFF\">$virtues[$count]</FONT></B></TD>\n",
                        "  <TD><IMG SRC=\"$root$loc{Five}$virtues{$virtues[$count]}$loc{Ext}\"></TD>\n",
		}
		else
		{ print "  <TD>&nbsp;</TD>\n  <TD>&nbsp;</TD>\n"; }

		# End of row
		print " </TR>\n";
	}

	# Additional Willpower/Humanity
	# Display Traits
	if ($ltraits > 0)
	{
	   # Graphic and headers
 	   print 	
		" <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{Base}\"><BR></TD></TR>\n\n",
		" <TR>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Other Traits</B></TD>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER>&nbsp;</TD>\n",
		"  <TD COLSPAN=\"2\" ALIGN=CENTER><B>Health</B></TD>\n",
		" </TR>\n";

	   print 
		" <TR>\n",
                "  <TD COLSPAN=\"1\" ALIGN=CENTER><B><FONT COLOR=\"#3F3FFF\">$traits[0]</FONT></B></TD>\n",
                "  <TD COLSPAN=\"1\" ALIGN=CENTER><IMG SRC=\"$root$loc{Ten}$traits{$traits[0]}$loc{Ext}\"></TD>\n",
                "  <TD COLSPAN=\"2\" ALIGN=CENTER ROWSPAN=\"$ltraits\">\n",
                "   <B>$pathtitle $pathname</B><BR>\n",
                "    <IMG SRC=\"$root$loc{Ten}$humanity$loc{Ext}\"><BR><BR>\n",
                "   <B>Willpower</B><BR>\n",
                "    <IMG SRC=\"$root$loc{Ten}$willpower$loc{Ext}\"><BR>\n",
                "    <IMG SRC=\"$root$loc{Willpower}\"><BR><BR>\n",
                "   <B>Blood Pool</B><BR>\n",
                "    <IMG SRC=\"$root$loc{Willpower}\"><BR>\n",
                "    <IMG SRC=\"$root$loc{Willpower}\"><BR>\n",
                "  </TD>\n",
                "  <TD COLSPAN=\"2\" ALIGN=CENTER ROWSPAN=\"$ltraits\"><IMG SRC=\"$root$loc{Health}\"></TD>\n",
                " </TR>\n";

                # Run through
                for ($count=1; $count < $ltraits; $count++)
		{
			print
			     " <TR>\n",
			     "  <TD COLSPAN=\"1\" ALIGN=CENTER><B><FONT COLOR=\"#3F3FFF\">$traits[$count]</FONT></B></TD>\n",
			     "  <TD COLSPAN=\"1\" ALIGN=CENTER><IMG SRC=\"$root$loc{Ten}$traits{$traits[$count]}$loc{Ext}\"></TD>\n",
			     " </TR>\n";  
		}
	}

	# No other traits
	else
	{
	   # Graphic and headers
 	   print 	
		" <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{Base}\"><BR></TD></TR>\n\n",
		" <TR>\n",
		"  <TD COLSPAN=\"3\" ALIGN=CENTER>&nbsp;</TD>\n",
		"  <TD COLSPAN=\"3\" ALIGN=CENTER><B>Health</B></TD>\n",
		" </TR>\n";

	  print
                " <TR>\n",
                "  <TD COLSPAN=\"3\" ALIGN=CENTER>\n",
                "   <B>$pathtitle $pathname</B><BR>\n",
                "    <IMG SRC=\"$root$loc{Ten}$humanity$loc{Ext}\"><BR><BR>\n",
                "   <B>Willpower</B><BR>\n",
                "    <IMG SRC=\"$root$loc{Ten}$willpower$loc{Ext}\"><BR>\n",
                "    <IMG SRC=\"$root$loc{Willpower}\"><BR><BR>\n",
                "   <B>Blood Pool</B><BR>\n",
                "    <IMG SRC=\"$root$loc{Willpower}\"><BR>\n",
                "    <IMG SRC=\"$root$loc{Willpower}\"><BR>\n",
                "  </TD>\n",
                "  <TD COLSPAN=\"3\" ALIGN=CENTER><IMG SRC=\"$root$loc{Health}\"></TD>\n",
                " </TR>\n";
	}	


	# Merits and Flaws
	# Determine lengths
	my $lmerits = @merits;
	my $lflaws  = @flaws;

	# Make sure they are defined
	$lmerits = defined($lmerits) ? $lmerits : 0; 
	$lflaws  = defined($lflaws)  ? $lflaws  : 0;
 
	# Determine longest
	$lmost   = ($lmerits > $lflaws) ? $lmerits : $lflaws;

	# Only show if there is something to show
	if (($lmerits != 0) || ($lflaws != 0))
	{
		# Graphic and headers
 		print 	" <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{Merits}\"><BR></TD></TR>\n\n",
			" <TR>\n",
			"  <TD COLSPAN=\"3\" ALIGN=CENTER><B>Merits</B></TD>\n",
			"  <TD COLSPAN=\"3\" ALIGN=CENTER><B>Flaws</B></TD>\n",
			" </TR>\n";

		# Run through
		for ($count=0; $count < $lmost; $count++)
		{
			# Beginning of row
			print " <TR>\n";

			if ($count < $lmerits)
			{ print "  <TD COLSPAN=\"3\" ALIGN=CENTER>$merits[$count]</TD>\n"; }
			else
			{ print "  <TD COLSPAN=\"3\" ALIGN=CENTER>&nbsp;</TD>\n"; }

			if ($count < $lflaws)
			{ print "  <TD COLSPAN=\"3\" ALIGN=CENTER>$flaws[$count]</TD>\n"; }
			else
			{ print "  <TD COLSPAN=\"3\" ALIGN=CENTER>&nbsp;</TD>\n"; }

			# End of row
			print " </TR>\n";
		}	
	}	


	# Derangements
	# Determine length
	my $lderangements = @derangements;
	my $lrituals      = @rituals;

	# Make sure they are defined
	$lderangements = defined($lderangements) ? $lderangements : 0; 
	$lrituals      = defined($lrituals)      ? $lrituals : 0;
 
	# Determine longest
	$lmost   = ($lrituals > $lderangements) ? $lrituals : $lderangements;


	# Only if any exists
	if (($lrituals != 0) || ($lderangements != 0))
	{
		# Graphic and headers
 		print 	" <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{Rituals}\"><BR></TD></TR>\n\n",
			" <TR>\n",
			"  <TD COLSPAN=\"3\" ALIGN=CENTER><B>Derangement</B></TD>\n",
			"  <TD COLSPAN=\"3\" ALIGN=CENTER><B>Rituals</B></TD>\n",
			" </TR>\n";

		# Run through
		for ($count=0; $count < $lmost; $count++)
		{
			# Beginning of row
			print " <TR>\n";

			if ($count < $lderangements)
			{ print "  <TD COLSPAN=\"3\" ALIGN=CENTER>$derangements[$count]</TD>\n"; }
			else
			{ print "  <TD COLSPAN=\"3\" ALIGN=CENTER>&nbsp;</TD>\n"; }

			if ($count < $lrituals)
			{ print "  <TD COLSPAN=\"3\" ALIGN=CENTER>$rituals[$count]</TD>\n"; }
			else
			{ print "  <TD COLSPAN=\"3\" ALIGN=CENTER>&nbsp;</TD>\n"; }

			# End of row
			print " </TR>\n";
		}	
	}	

	# Combat bar
	print  " <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{CombatBar}\"><BR></TD></TR>\n\n",
	       " <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{Combat}\"><BR></TD></TR>\n\n";



	# Background/History
	# Graphic and headers
	print 	" <TR><TD COLSPAN=\"6\" ALIGN=CENTER><BR><IMG SRC=\"$root$loc{Base}\"><BR></TD></TR>\n\n",
		" <TR>\n",
		"  <TD COLSPAN=\"6\" ALIGN=CENTER><B>Background/History</B></TD>\n",
		" </TR>\n";

	# Only show picture of need be
	if (defined($picture) && (length($picture) > 5))
	{
		print	" <TR>\n",
			"  <TD COLSPAN=\"6\" ALIGN=CENTER><IMG SRC=\"$picture\"></TD>\n",
			" </TR>\n";
	}

	print	" <TR>\n",
		"  <TD COLSPAN=\"6\" ALIGN=CENTER>$history</TD>\n",
		" </TR>\n";

	# End of Table
	print	"</TABLE></CENTER>\n";


	# Notice of copyright
	print "<BR>\n<BR>\n";
	print "<P><CENTER>All sheet logos are copyrighted by <A HREF=\"http://www.white-wolf.com\">White Wolf Publishing, Inc.</A></CENTER><P>\n";

	# End of HTML
	print $cgi->end_html();
